﻿Public Class HSV
    ' All values are between 0 and 255.

    ' Unlike in the RGB class, there's no manipulation 
    ' necessary on these values, so it's easiest to just leave
    ' these as public fields. Easy enough to just change them
    ' to property procedures, if you need to add functionality.
    Public H As Integer
    Public S As Integer
    Public V As Integer

    Public Sub New( _
     ByVal H As Integer, ByVal S As Integer, ByVal V As Integer)
        Me.H = H
        Me.S = S
        Me.V = V
    End Sub

    Public Overrides Function ToString() As String
        Return String.Format("({0}, {1}, {2})", H, S, V)
    End Function

#Region " Overloaded Operators "
    Public Overloads Shared Narrowing Operator CType( _
     ByVal value As HSV) As Color
        ' Require explicit conversion from HSV to Color.
        ' Take advantage of implicit conversion from
        ' RGB to Color.
        If value Is Nothing Then
            Return Nothing
        Else
            Return ToRGB(value)
        End If
    End Operator

    Public Overloads Shared Narrowing Operator CType( _
     ByVal value As Color) As HSV
        ' Require explicit conversions from Color to HSV.
        Dim RGB As New RGB(value)
        Return CType(RGB, HSV)
    End Operator

    Public Overloads Shared Narrowing Operator CType( _
     ByVal value As HSV) As RGB
        ' Require explicit conversion (using CType)
        ' from HSV to RGB.
        If value Is Nothing Then
            Return Nothing
        Else
            Return ToRGB(value)
        End If
    End Operator

    Public Overloads Shared Operator =( _
     ByVal value1 As HSV, ByVal value2 As HSV) As Boolean
        ' Compare two colors, either of which might be Nothing.

        If value1 Is Nothing And value2 Is Nothing Then
            ' If both values are Nothing, return True--the colors are the same.
            Return True
        ElseIf value1 Is Nothing Then
            Return False
        Else
            Return value1.Equals(value2)
        End If
    End Operator

    Public Overloads Shared Operator =(ByVal value1 As HSV, ByVal value2 As Color) As Boolean
        ' Compare an HSV and a Color. 
        If value1 Is Nothing Then
            ' A Color instance can't be null, so if the HSV
            ' is Nothing, return False.
            Return False
        Else
            Return value1.Equals(value2)
        End If
    End Operator

    Public Overloads Shared Operator =(ByVal value1 As HSV, ByVal value2 As RGB) As Boolean
        If value1 Is Nothing And value2 Is Nothing Then
            Return True
        ElseIf value1 Is Nothing Then
            Return False
        Else
            Return value1.Equals(value2)
        End If
    End Operator

    Public Overloads Shared Operator <>(ByVal value1 As HSV, ByVal value2 As Color) As Boolean
        Return Not (value1 = value2)
    End Operator

    Public Overloads Shared Operator <>(ByVal value1 As HSV, ByVal value2 As HSV) As Boolean
        Return Not (value1 = value2)
    End Operator

    Public Overloads Shared Operator <>(ByVal value1 As HSV, ByVal value2 As RGB) As Boolean
        Return Not (value1 = value2)
    End Operator

#End Region

#Region " Overloaded Methods "

    Public Overloads Function Equals(ByVal value As HSV) As Boolean
        If value Is Nothing Then
            ' If value is Nothing, the comparison must fail--the current
            ' instance cannot be null, so the two can't be equal.
            Return False
        Else
            Return (Me.H = value.H AndAlso _
             Me.S = value.S AndAlso Me.V = value.V)
        End If
    End Function

    Public Overloads Function Equals(ByVal value As Color) As Boolean
        ' The current instance can't be Nothing, and the Color
        ' can't be Nothing, so there's no need to check for
        ' null references.

        ' Convert this instance into an RGB value for comparison.
        Dim rgbValue As New RGB(value)
        Return rgbValue.Equals(Me)
    End Function

    Public Overloads Function Equals(ByVal value As RGB) As Boolean
        If value Is Nothing Then
            Return False
        Else
            Return value.Equals(Me)
        End If
    End Function

    Public Overloads Function GetHashCode() As Integer
        ' Need to provide an arbitrary integer
        ' that represents this color. 
        Return CType(Me, System.Drawing.Color).GetHashCode
    End Function
#End Region


    Private Shared Function ToRGB(ByVal HSV As HSV) As RGB
        ' HSV contains values scaled as in the color wheel:
        ' that is, all from 0 to 255. 

        ' For this code to work, HSV.Hue needs
        ' to be scaled from 0 to 360 (it's the angle of the selected
        ' point within the circle). HSV.Saturation and HSV.Value must be 
        ' scaled to be between 0 and 1.

        Dim h As Decimal
        Dim s As Decimal
        Dim v As Decimal

        Dim r As Decimal
        Dim g As Decimal
        Dim b As Decimal

        ' Scale Hue to be between 0 and 360. Saturation
        ' and Value scale to be between 0 and 1.
        h = (HSV.H / 255D * 360) Mod 360
        s = HSV.S / 255D
        v = HSV.V / 255D

        If s = 0 Then
            ' If s is 0, all colors are the same.
            ' This is some flavor of gray.
            r = v
            g = v
            b = v
        Else
            Dim p As Decimal
            Dim q As Decimal
            Dim t As Decimal

            Dim fractionalSector As Decimal
            Dim sectorNumber As Integer
            Dim sectorPos As Decimal

            ' The color wheel consists of 6 sectors.
            ' Figure out which sector you're in.
            sectorPos = h / 60
            sectorNumber = CInt(Math.Floor(sectorPos))

            ' Get the fractional part of the sector.
            ' That is, how many degrees into the sector
            ' are you?
            fractionalSector = sectorPos - sectorNumber

            ' Calculate values for the three axes
            ' of the color. 
            p = v * (1 - s)
            q = v * (1 - (s * fractionalSector))
            t = v * (1 - (s * (1 - fractionalSector)))

            ' Assign the fractional colors to r, g, and b
            ' based on the sector the angle is in.
            Select Case sectorNumber
                Case 0
                    r = v
                    g = t
                    b = p

                Case 1
                    r = q
                    g = v
                    b = p

                Case 2
                    r = p
                    g = v
                    b = t

                Case 3
                    r = p
                    g = q
                    b = v

                Case 4
                    r = t
                    g = p
                    b = v

                Case 5
                    r = v
                    g = p
                    b = q
            End Select
        End If
        ' Return an RGB instance, with values scaled
        ' to be between 0 and 255.
        Return New RGB(CInt(r * 255), CInt(g * 255), CInt(b * 255))

    End Function
End Class